using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Security.Cryptography;
using ldap_cesi.Context;
using ldap_cesi.Entities;
using ldap_cesi.Services.Interfaces;
using Microsoft.IdentityModel.Tokens;

namespace ldap_cesi.Services;

public class JwtService : IJwtService
{
    private readonly IRsaKeyService _rsaKeyService;
    private readonly IConfiguration _configuration;
    private readonly PgContext _context;
    private readonly ILogger<JwtService> _logger;

    public JwtService(
        IConfiguration configuration,
        PgContext context,
        IRsaKeyService rsaKeyService,
        ILogger<JwtService> logger)
    {
        _configuration = configuration;
        _context = context;
        _rsaKeyService = rsaKeyService;
        _logger = logger;
    }

    public string GenerateToken(Utilisateur utilisateur)
    {
        try
        {
            var claims = new[]
            {
                new Claim(JwtRegisteredClaimNames.Sub, utilisateur.Email),
                new Claim(JwtRegisteredClaimNames.Jti, Guid.NewGuid().ToString()),
                new Claim(ClaimTypes.Name, utilisateur.Nom),
                new Claim(ClaimTypes.Role, utilisateur.IdRoleNavigation.Nom),
                new Claim(ClaimTypes.NameIdentifier, utilisateur.Id.ToString())
            };

            var key = new RsaSecurityKey(_rsaKeyService.GetRsaKey());
            var creds = new SigningCredentials(key, SecurityAlgorithms.RsaSha256);

            var tokenExpiryMinutes = _configuration.GetValue<int>("Jwt:TokenExpiryMinutes", 30);
            var token = new JwtSecurityToken(
                issuer: _configuration["Jwt:Issuer"],
                audience: _configuration["Jwt:Audience"],
                claims: claims,
                expires: DateTime.UtcNow.AddMinutes(tokenExpiryMinutes),
                signingCredentials: creds);

            var tokenString = new JwtSecurityTokenHandler().WriteToken(token);
            utilisateur.AccessToken = tokenString;
            _context.Utilisateurs.Update(utilisateur);
            _context.SaveChanges();

            return tokenString;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "An error occurred while generating the JWT token.");
            throw;
        }
    }

    public string GetPublicKey()
    {
        try
        {
            var publicKey = _rsaKeyService.GetRsaKey().ExportSubjectPublicKeyInfo();
            return Convert.ToBase64String(publicKey);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "An error occurred while retrieving the public key.");
            throw;
        }
    }

    public async Task<bool> ValidateToken(string token, int userId)
    {
        try
        {
            var tokenHandler = new JwtSecurityTokenHandler();
            var validationParameters = new TokenValidationParameters
            {
                ValidateIssuerSigningKey = true,
                IssuerSigningKey = new RsaSecurityKey(_rsaKeyService.GetRsaKey()),
                ValidateIssuer = true,
                ValidateAudience = true,
                ValidIssuer = _configuration["Jwt:Issuer"],
                ValidAudience = _configuration["Jwt:Audience"],
                ClockSkew = TimeSpan.Zero,
                ValidateLifetime = true
            };

            var principal = tokenHandler.ValidateToken(token, validationParameters, out var validatedToken);
            var nameIdClaim = principal.FindFirst(ClaimTypes.NameIdentifier);
            if (nameIdClaim == null || !int.TryParse(nameIdClaim.Value, out var tokenUserId) || tokenUserId != userId)
            {
                _logger.LogWarning("Token validation failed: Invalid user ID.");
                return false;
            }

            var utilisateur = await _context.Utilisateurs.FindAsync(userId);
            if (utilisateur == null)
            {
                _logger.LogWarning("Token validation failed: User not found.");
                return false;
            }

            utilisateur.AccessToken = token;
            _context.Utilisateurs.Update(utilisateur);
            await _context.SaveChangesAsync();

            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "An error occurred while validating the JWT token.");
            return false;
        }
    }
}